@@ -1,6 +1,8 @@
 #ifndef LIBGNOMEUI_H
 #define LIBGNOMEUI_H
 
+#include "libgnomeui/gnome-compat.h"
+
 #include "libgnome/gnome-defs.h"
 #include "libgnomeui/gnome-uidefs.h"
 #include "libgnomeui/gnome-about.h"
@@ -12,6 +14,7 @@
 #include "libgnomeui/gnome-canvas.h"
 #include "libgnomeui/gnome-canvas-image.h"
 #include "libgnomeui/gnome-canvas-line.h"
+#include "libgnomeui/gnome-canvas-load.h"
 #include "libgnomeui/gnome-canvas-rect-ellipse.h"
 #include "libgnomeui/gnome-canvas-polygon.h"
 #include "libgnomeui/gnome-canvas-text.h"
