@@ -237,7 +237,10 @@ _gnome_gnorba_cookie_setup(Display *disp, Window rootwin)
   if(ret_type == None)
     ret_prop = NULL;
   else
-    XUngrabServer(disp);
+    {
+      XUngrabServer(disp);
+      XFlush(disp);
+    }
 
   setval = _gnorba_cookie_setup(ret_prop);
 
@@ -245,11 +248,9 @@ _gnome_gnorba_cookie_setup(Display *disp, Window rootwin)
     XChangeProperty(disp, rootwin, prop, XA_STRING, 8, PropModeReplace,
 		    setval, strlen(setval));
     XUngrabServer(disp);
-  }
-
-  if(ret_prop) {
+    XFlush(disp);
+  } else {
     XFree(ret_prop); /* XFree barfs on NULL ptrs */
-    XUngrabServer(disp);
   }
 }
 
@@ -274,6 +275,7 @@ gnorba_CORBA_init(int *argc, char **argv,
 		  CORBA_Environment *ev)
 {
 	CORBA_ORB retval;
+	gchar *cookie;
 
 #ifndef ORBIT_USES_GLIB_MAIN_LOOP
 	IIOPAddConnectionHandler = orb_add_connection;
@@ -282,8 +284,10 @@ gnorba_CORBA_init(int *argc, char **argv,
 
 	_gnorba_gnome_orbit_orb = retval = CORBA_ORB_init(argc, argv, "orbit-local-orb", ev);
 	
-	if(!(flags & GNORBA_INIT_DISABLE_COOKIES))
-	  _gnorba_cookie_setup(NULL);
+	if(!(flags & GNORBA_INIT_DISABLE_COOKIES)) {
+	  char *cookie = _gnorba_cookie_setup(NULL);
+	  g_free (cookie);
+	}
 	
 	return retval;
 }
