@@ -205,7 +205,7 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
 	((c & (VTATTR_MASK^VTATTR_FORECOLOURM)) != 
 	 (l->data[i] & (VTATTR_MASK^VTATTR_FORECOLOURM)))) {
 
-#if 1
+#if 0
       d(printf("bl->data[i] = %08x != %08x, c=%d (back now=%d not %d)\n", 
 	       bl->data[i], l->data[i], c, 
 	       (l->data[i]&VTATTR_BACKCOLOURM)>>VTATTR_BACKCOLOURB, 
@@ -214,7 +214,7 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
       vx->back_match = 0;
       break;
     } else {
-#if 1
+#if 0
       d(printf("bl->data[i] == %08x == %08x, c=%08x back=%d\n",
 	       bl->data[i], l->data[i], c,
 	       (l->data[i]&VTATTR_BACKCOLOURM)>>VTATTR_BACKCOLOURB));
@@ -261,6 +261,7 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
       if (attr != newattr) { /* check run of same type ... */
 	d(printf("found a run of %d characters from %d: '", run, runstart));
 	d(fwrite(vx->runbuffer, run, 1, stdout));
+	d(printf("\n"));
 	vt_draw_text(vx->vt.user_data, runstart, line, vx->runbuffer, run, attr);
 	runstart = i;
 	p = vx->runbuffer;
